@@ -2,7 +2,7 @@
 # ==============================================================================
 # portsreinstall library script
 # - Temporary directory and signal trapping -
-# Copyright (C) 2013-2015 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2013-2016 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
@@ -16,16 +16,16 @@ temp_terminate_process () { :; }
 temp_warn_obsolete_temp_db ()
 {
 	[ "x${DATABASE_IS_OBSOLETE}" = xyes ] || return 0
-	message_echo "WARNING: The Ports tree was updated after construction of the temporary database for ${APPNAME}." >&2
-	message_echo "You should consider executing " >&2
-	message_echo "        ${APPNAME} reset" >&2
-	message_echo "to reset the temporary database unless you have special purposes." >&2
+	message_echo "WARNING: The Ports tree was updated after construction of the temporary database for ${APPNAME}."
+	message_echo "You should consider executing "
+	message_echo "        ${APPNAME} reset"
+	message_echo "to reset the temporary database unless you have special purposes."
 }
 
 # ============= Creation of temporary work directories =============
 temp_trap_init ()
 {
-	trap 'errno=$?; temp_warn_obsolete_temp_db; temp_warn_obsolete_temp_db () { :; }; temp_terminate_process; temp_terminate_process () { :; }; rm -rf "${TMPDIR}" 2> /dev/null; [ -n "${DBDIR}" -a x`cat "${DBDIR}/.lock" 2> /dev/null` = x$$ ] && rm -f "${DBDIR}/.lock"; [ $errno -gt 0 -a $errno -ne 130 ] && message_echo "(Error exit by $errno)" >&2; exit $errno' 0 1 2 3 9 15 17 18
+	trap 'errno=$?; temp_warn_obsolete_temp_db >&2; temp_warn_obsolete_temp_db () { :; }; temp_terminate_process >&2; temp_terminate_process () { :; }; rm -rf "${TMPDIR}" 2> /dev/null; [ -n "${DBDIR}" -a x`cat "${DBDIR}/.lock" 2> /dev/null` = x$$ ] && rm -f "${DBDIR}/.lock"; [ $errno -gt 0 -a $errno -ne 130 ] && message_echo "(Error exit by $errno)" >&2; exit $errno' 0 1 2 3 9 15 17 18
 	{ until TMPDIR=`mktemp -dq /tmp/"${APPNAME}".XXXXXXXX` ; do : ; done ; }
 	chgrp "`id -gn`" "${TMPDIR}"
 }
